package com.fulan.application.handle.quartz.job;

import com.fulan.application.feign.CourseStudyHourRankingClient;
import com.fulan.application.handle.quartz.BaseJob;
import com.fulan.application.system.remote.SiteClient;
import org.quartz.InterruptableJob;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.UnableToInterruptJobException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * 课程学习时长排名处理
 * <br>  第二步骤START_TIME 时间初始化为0，表示job不会触发，等功能实现完需要填写开始启动的时间戳
 * <br>  如果需要在job的corn之前再次启动可以修改第二步的NEXT_FIRE_TIME 字段为下次启动的时间戳
 *
 */
public class CourseStudyHourRankingJob implements BaseJob, InterruptableJob {

    private final Logger LOG = LoggerFactory.getLogger(CourseStudyHourRankingJob.class);

    //feature_867_xiehaijun
    @Autowired
    private CourseStudyHourRankingClient courseStudyHourRankingClient;

    @Autowired
    private SiteClient siteClient;

    // 参考http://www.ibloger.net/article/2650.html

    // 1 在QRTZ_JOB_DETAILS 配置 job执行的处理类，实现 implements BaseJob, InterruptableJob
    //SCHED_NAME  JOB_NAME	    JOB_GROUP	  DESCRIPTION	  JOB_CLASS_NAME	                                                 IS_DURABLE	 IS_NONCONCURRENT	IS_UPDATE_DATA	REQUESTS_RECOVERY
    //scheduler	  studyHourRanking	course	  课程学习时间排名	Job定义  com.fulan.application.handle.quartz.job.CourseStudyHourRankingJob	 0	          0	               0	             0

    // 2 在QRTZ_TRIGGERS（存储已配置的 Trigger 的信息）表添加触发时间点（每次执行完批处理后，根据QRTZ_CRON_TRIGGERS表配置的cron表达式生成新的下一个执行时间）
            // NEXT_FIRE_TIME（下次执行时间戳例如1566282600000）和NEXT_FIRE_TIME设置为0；为0表示初始化，这些字段在下个job启动完会更新
            // NEXT_FIRE_TIME的更新：在暂停、恢复任务状态或是每次时间轮询时都会判断NEXT_FIRE_TIME值是否小于当前时间，如果小于当前时间将永远不会执行
    //SCHED_NAME TRIGGER_NAME	        TRIGGER_GROUP JOB_NAME	        JOB_GROUP DESCRIPTION	        NEXT_FIRE_TIME	PREV_FIRE_TIME	PRIORITY	TRIGGER_STATE	TRIGGER_TYPE  START_TIME	   END_TIME	CALENDAR_NAME	MISFIRE_INSTR
    //scheduler	 studyHourRankingTrig	courseTrig	  studyHourRanking	course	  课程学习时间排名触发器	0	            0		           null      WAITING	     CRON	      填写开始启动时间戳  0			null            null


    // 3 在QRTZ_CRON_TRIGGERS 表配置cron表达式
    //SCHED_NAME	TRIGGER_NAME	        TRIGGER_GROUP	CRON_EXPRESSION	 TIME_ZONE_ID
    //scheduler	    studyHourRankingTrig	courseTrig	    0 0 4 * * ?	     Asia/Shanghai


    @Override
    public void execute(JobExecutionContext context) throws JobExecutionException {

        LOG.info("######################################################课程学习时长排名生成处理开始");
        List<Long> siteIds = siteClient.getAllSiteId();

        for(Long id : siteIds){
            LOG.info("站点{}开始生成站点排名......",id);
            try {
                TimeUnit.SECONDS.sleep(5);
            } catch (InterruptedException e) {
                LOG.info("等待发生一次{}", e);
            }
            courseStudyHourRankingClient.rankingGenerated(id);
        }
    }

    @Override
    public void interrupt() throws UnableToInterruptJobException {

    }
}
