package com.fulan.application.handle.quartz.job;

import cn.hutool.core.date.DateUtil;
import com.fulan.application.feign.RemoteDaIndicatorCourseClient;
import com.fulan.application.feign.RemoteDaIndicatorTpClient;
import com.fulan.application.handle.quartz.BaseJob;
import com.fulan.application.system.remote.RemoteDaIndicatorSystemClient;
import com.fulan.application.system.remote.SiteClient;
import com.fulan.application.vo.RemoteDaIndicatorVo;
import org.apache.commons.collections.CollectionUtils;
import org.quartz.InterruptableJob;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.UnableToInterruptJobException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Date;
import java.util.List;

/**
 * 首页dashboard指标job
 */
public class DashboardJob implements BaseJob, InterruptableJob {

    private static final Logger log = LoggerFactory.getLogger(DashboardJob.class);

    @Autowired
    private RemoteDaIndicatorSystemClient systemClient;
    @Autowired
    private RemoteDaIndicatorCourseClient courseClient;
    @Autowired
    private RemoteDaIndicatorTpClient tpClient;
    @Autowired
    private SiteClient siteClient;

    @Override
    public void execute(JobExecutionContext context) throws JobExecutionException {
        List<Long> allSiteIds = siteClient.getAllSiteId();
        if (CollectionUtils.isEmpty(allSiteIds)) {
            log.info("-------------------- dashboard 未查询到站点列表，结束执行 ----------------------");
            return;
        }

        Date now = new Date();
        Date yesterday = DateUtil.offsetDay(now, -1).toJdkDate();
        Date endOfWeek = DateUtil.beginOfDay(DateUtil.endOfWeek(yesterday)).toJdkDate();
        Date startOfWeek = DateUtil.beginOfDay(DateUtil.beginOfWeek(yesterday)).toJdkDate();

        allSiteIds.forEach(siteId -> {
            try {
                log.info("-------------------- dashboard 折线图 开始处理 site_id: {} ----------------------", siteId);
                systemClient.accountDay(new RemoteDaIndicatorVo(siteId, null, null, yesterday, now));
                log.info("-------------------- dashboard 折线图 处理完成 site_id: {} ----------------------", siteId);
            } catch (Exception e) {
                e.printStackTrace();
            }

            // 如果是最后一天，则统计dashboard4个指标
            if (DateUtil.isSameDay(yesterday, endOfWeek)) {
                try {
                    log.info("-------------------- dashboard 活跃用户比例 开始处理 site_id: {} ----------------------", siteId);
                    systemClient.account(new RemoteDaIndicatorVo(siteId, startOfWeek, endOfWeek, null, now));
                    log.info("-------------------- dashboard 活跃用户比例 处理完成 site_id: {} ----------------------", siteId);
                } catch (Exception e) {
                    e.printStackTrace();
                }

                try {
                    log.info("-------------------- dashboard 课程完成数 开始处理 site_id: {} ----------------------", siteId);
                    courseClient.courseFinish(new RemoteDaIndicatorVo(siteId, startOfWeek, endOfWeek, null, now));
                    log.info("-------------------- dashboard 课程完成数 处理完成 site_id: {} ----------------------", siteId);
                } catch (Exception e) {
                    e.printStackTrace();
                }

                try {
                    log.info("-------------------- dashboard 课程学习时长 开始处理 site_id: {} ----------------------", siteId);
                    courseClient.courseTime(new RemoteDaIndicatorVo(siteId, startOfWeek, endOfWeek, null, now));
                    log.info("-------------------- dashboard 课程学习时长 处理完成 site_id: {} ----------------------", siteId);
                } catch (Exception e) {
                    e.printStackTrace();
                }

                try {
                    log.info("-------------------- dashboard 项目通过率 开始处理 site_id: {} ----------------------", siteId);
                    tpClient.tpFinish(new RemoteDaIndicatorVo(siteId, startOfWeek, endOfWeek, null, now));
                    log.info("-------------------- dashboard 项目通过率 处理完成 site_id: {} ----------------------", siteId);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

    @Override
    public void interrupt() throws UnableToInterruptJobException {

    }
}
