/**
 * FileName: HttpSetjob
 * Author:   wenjunlong
 * Date:     2018/5/26 13:59
 * Description: 发送请求job
 * History:
 * <author>          <time>          <version>          <desc>
 * 作者姓名           修改时间           版本号              描述
 */
package com.fulan.application.handle.quartz.job;

import com.alibaba.fastjson.JSON;
import com.fulan.application.cache.RedisCache;
import com.fulan.application.domain.*;
import com.fulan.application.event.EventWrapper;
import com.fulan.application.feign.*;
import com.fulan.application.handle.quartz.BaseJob;
import com.fulan.application.publish.CloudEventPublisher;
import com.fulan.application.task.TaskExecutor;
import com.fulan.application.util.constant.QueueConstant;
import com.fulan.application.util.event.TrainingProjectEvent;
import com.fulan.application.vo.PointParamVO;
import com.fulan.application.vo.RecordeDurationVO;
import org.apache.commons.lang3.time.DateUtils;
import org.quartz.InterruptableJob;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.UnableToInterruptJobException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.Calendar;

/**
 * 〈一句话功能简述〉<br>
 * 〈发送请求job〉
 * 实时监控学员学习记录更新课程完成状态，学习记录存放在study_log表中
 * 课程学习页面会每隔1秒调用heart_check接口，记录学习日志信息，通过SQL脚本把学习记录添加到study_log表中，该脚本是系统运维管理员编写的，不在项目中
 * 然后通过定时分析学习日志，更新课程完成的状态
 *
 * @author wenjunlong
 * @create 2018/5/26
 * @since 1.0.0
 */
public class HttpSetjobBack implements BaseJob, InterruptableJob {

    private final Logger LOG = LoggerFactory.getLogger(HttpSetjobBack.class);

    /**
     * 所有的课程 Map: courseId -> course
     */
    public Map<Long, Course> allCoursesMap = new HashMap<>();
    /**
     * 所有的章节 Map: chapterId -> chapter
     */
    public Map<Long, Chapter> allChaptersMap = new HashMap<>();
    /**
     * 所有的素材 Map: materialId -> material
     */
    public Map<Long, Material> allMaterialsMap = new HashMap<>();
    /**
     * 所有的学习记录 Map: sessionKey(accountId+courseId+chapterId) -> record
     */
    public Map<String, Recorde> allRecordeMap = new HashMap<>();
    /**
     * 所有学员的每个章节的总学习时长 Map: key(accountId+chapterId) -> 章节学习时长
     */
    public Map<String, Integer> allRecordeDurationMap = new HashMap<>();

    @Autowired
    RecordeClient recordeClient;
    @Autowired
    StudyLogClient studyLogClient;
    @Autowired
    PointRedisFeignClients pointRedisFeignClients;
    @Autowired
    AmqpTemplate amqpTemplate;
    @Autowired
    CourseClient courseClient;
    @Autowired
    MaterialClient materialClient;
    @Autowired
    RedisCache redisCache;
    @Autowired
    private TaskExecutor taskExecutor;
    @Autowired
    private CloudEventPublisher cloudEventPublisher;
    @Autowired
    private CourseStudyClient courseStudyClient;

    /**
     * 初始化 课程 章节 素材 学习记录list
     */
    private void init() {
        List<Course> courseList = recordeClient.getCourseList();
        LOG.info("心跳：setp1");
        for (Course course : courseList) {
            this.allCoursesMap.put(course.getId(), course);
        }

        List<Chapter> chapterList = recordeClient.getChapterList();
        LOG.info("心跳：setp2");
        for (Chapter chapter : chapterList) {
            this.allChaptersMap.put(chapter.getId(), chapter);
        }
        List<Material> materialList = recordeClient.getMaterialList();
        LOG.info("心跳：setp3");
        for (Material material : materialList) {
            this.allMaterialsMap.put(material.getId(), material);
        }

        List<Recorde> recordes = recordeClient.getAllRecordeList();
        LOG.info("心跳：setp4");
        for (Recorde recorde : recordes) {
            String sessionkey = makeSessionKey(recorde);
            // 如果map 中已经有该记录，并且是已完成该章节
            // 如果map 中已经有该记录，并且是已完成该章节
            Recorde existRecorde = allRecordeMap.get(sessionkey);// 解决空指针异常问题
            //LOG.info("existRecorde={}",existRecorde);
            int existOvered = 0;
            if(existRecorde != null && existRecorde.getOvered()!= null){
                existOvered = existRecorde.getOvered();
            }
            if (allRecordeMap.containsKey(sessionkey) && existOvered ==1) {
                continue;
            }
            this.allRecordeMap.put(sessionkey, recorde);
        }

        List<RecordeDurationVO> recordeDurationVOS = recordeClient.getRecordeDuration();
        for (RecordeDurationVO recordeDurationVO : recordeDurationVOS) {
            if (recordeDurationVO.getChapterId() == null || recordeDurationVO.getAccountId() == null) {
                continue;
            }
            this.allRecordeDurationMap.put(recordeDurationVO.getAccountId().toString() + recordeDurationVO.getChapterId().toString(), recordeDurationVO.getDuration());
        }
        LOG.info("心跳：setp5");
    }

    @Override
    public void execute(JobExecutionContext context) throws JobExecutionException {
        try {
            long start = System.currentTimeMillis();
            LOG.info("<---------------------------------定时解析心跳---------------------------------------------------------->" + start);
            List<StudyLog> studyLogs = studyLogClient.list("");

            if (studyLogs.isEmpty() & studyLogs.size() == 0) {
                LOG.info("无学习记录");
            } else {

                // 初始化 课程 章节 素材 学习记录 等数据
                init();

                // 1. 将学习记录日志整理成学习记录 用于判断学习记录的完成状态

                // 当前获取的新学习记录 Map：sessionKey -> record
                Map<String, Recorde> currentRecordeMap = new HashMap<String, Recorde>();
                List<Long> ids = new ArrayList<>();

                // 每条 studyLog 的记录时间
                Date newTime = null;
                for (StudyLog studyLog : studyLogs) {
                    ids.add(studyLog.getId());
                    String sessionKey = makeSessionKey(studyLog);
                    // 如果当前 学习记录 map 中已经有过该章节的学习记录信息，更新开始时间（算最小的那条）、结束时间（算最大的那条）、是否完成（完成一次就算完成）
                    if (currentRecordeMap.containsKey(sessionKey)) {
                        Recorde recorde = currentRecordeMap.get(sessionKey);
                        newTime = new Date(studyLog.getTimestamp());
                        // 如果 record 的结束时间 < newTime，设为新的结束时间
                        if (DateUtils.truncatedCompareTo(recorde.getEndTime(), newTime, Calendar.MILLISECOND) < 0) {
                            recorde.setEndTime(newTime);
                        }
                        // 如果 record 的开始时间 > newTime，设为新的开始时间
                        if (DateUtils.truncatedCompareTo(recorde.getStartTime(), newTime, Calendar.MILLISECOND) > 0) {
                            recorde.setStartTime(newTime);
                        }
                        if (recorde.getOvered() == 0) {
                            recorde.setOvered(studyLog.getMaterialCompleted());
                        }
                    }
                    // 如果当前 学习记录 map 中没有章节的学习记录信息，组装一条 Record
                    else {
                        Recorde recorde = new Recorde();
                        if (!StringUtils.isEmpty(studyLog.getAccountId())) {
                            recorde.setAccountId(stringToLong(studyLog.getAccountId()));
                        }
                        if (!StringUtils.isEmpty(studyLog.getCompanyId())) {
                            recorde.setCompanyId(stringToLong(studyLog.getCompanyId()));
                        }
                        if (!StringUtils.isEmpty(studyLog.getOrgId())) {
                            recorde.setOrgId(stringToLong(studyLog.getOrgId()));
                        }
                        recorde.setChapterId(studyLog.getChapterId());
                        recorde.setCourseId(studyLog.getCourseId());
                        recorde.setStartTime(new Date(studyLog.getTimestamp()));
                        recorde.setEndTime(new Date(studyLog.getTimestamp()));
                        recorde.setTerminalDevice(studyLog.getTerminalDevice());
                        recorde.setTerminalType(studyLog.getTerminalType());
                        recorde.setOvered(studyLog.getMaterialCompleted());
                        recorde.setSiteId(studyLog.getSiteId());
                        recorde.setSessionId(studyLog.getSessionId());
                        recorde.setOvered(studyLog.getMaterialCompleted());

                        //把学习记录 加上当前课程的企业id 站点id 部门id
                        if (allCoursesMap.containsKey(recorde.getCourseId())) {
                            Course course = allCoursesMap.get(recorde.getCourseId());
                            recorde.setSiteId(course.getSiteId());
                            recorde.setOrgId(course.getOrgId());
                            recorde.setCompanyId(course.getCompanyId());
                        }

                        currentRecordeMap.put(sessionKey, recorde);
                    }
                    // 设置为已经解析过的日志
                    studyLog.setStatus(1);
                }
                LOG.info("心跳：setp6");
                Date nowDate = new Date();

                // 将要保存的新纪录
                List<Recorde> newRecords = new ArrayList<>();

                // 循环本次新产生的学习记录 recordMap
                for (Map.Entry<String, Recorde> entry : currentRecordeMap.entrySet()) {
                    Recorde recorde = entry.getValue();
                    recorde.setValid(1);
                    recorde.setExitstates(1);
                    recorde.setCreateTime(nowDate);

                    Long duration = 0L;
                    duration = recorde.getEndTime().getTime() / 1000 - recorde.getStartTime().getTime() / 1000 + 5;
                    recorde.setDuration(duration.intValue());

                    //查询先前是否有未完成的记录
                    String sessionkey = makeSessionKey(recorde);
                    Recorde oldRecord = allRecordeMap.get(sessionkey);

                    // 如果之前没有该章节的学习记录
                    if (oldRecord == null || StringUtils.isEmpty(oldRecord.getId())) {
                        if (0 == recorde.getOvered()) {
                            Integer completed = getCompleted(recorde.getChapterId(), recorde.getAccountId(), recorde.getDuration());
                            recorde.setOvered(completed);
                        }
                    }
                    // 之前就已经有该章节的学习记录
                    else {
                        int newOvered = recorde.getOvered()==null?0:recorde.getOvered();
                        int oldOvered = oldRecord.getOvered()==null?0:oldRecord.getOvered();
                        if (0 == newOvered && 0 == oldOvered) {
                            Integer completed = getCompleted(oldRecord.getChapterId(), oldRecord.getAccountId(), oldRecord.getDuration());
                            recorde.setOvered(completed);
                        } else {
                            oldRecord.setOvered(1);
                        }
                    }
                    newRecords.add(recorde);
                }
                LOG.info("心跳：setp7");
                // 插入本次解析到的学习记录
                try {
                    if (!CollectionUtils.isEmpty(newRecords)) {
                        newRecords = recordeClient.saveList(newRecords);
                    }
                } catch (Exception e) {
                    LOG.error("请求失败" + e);
                    LOG.error("执行学习记录操作回滚");
                    if (newRecords.size() > 0) {
                        recordeClient.deleteList(newRecords);
                    }
                    return;
                }

                // 删除本次解析过的学习日志
                boolean flag = studyLogClient.deleteBatch(ids);
                if (!flag) {
                    return;
                }
                LOG.info("心跳：setp8");
                // 发放积分
                for (Recorde recorde : newRecords) {
                    if (1 == recorde.getOvered()) {
                        //getPoint(recorde.getChapterId(), recorde.getAccountId());
                        try {
                            courseStudyClient.completionStatus(recorde);
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                }

                LOG.info("心跳：setp9");
                // 发送课程章节完成状态
                LOG.info("开始准备发送课程完成状态处理");
                sendCourseFinishedMsg(newRecords);

                long end = System.currentTimeMillis();
                LOG.info("学习日志解析成功，并且学习记录更新成功");
                LOG.info("<---------------------------------End---------------------------------------------------------->");
                LOG.info("经过时间" + (end - start));
            }
        } catch (Exception e) {
            LOG.info(JSON.toJSONString(e));
            LOG.info("<---------------------------------执行异常---------------------------------------------------------->");
            e.printStackTrace();
            LOG.error("请求失败" + e);
            LOG.info("<---------------------------------End---------------------------------------------------------->");

        }
    }

    /**
     * 处理发送培训项目完成信息
     *
     * @param newRecords 
     */
    private void sendCourseFinishedMsg(List<Recorde> newRecords) {
        Map<String, List<Recorde>> records = new HashMap<>();
        String key = null;
        for (Recorde r : newRecords) {
            key = r.getCourseId() + "_" + r.getAccountId();
            if (records.containsKey(key)) {
                records.get(key).add(r);
            } else {
                List<Recorde> list = new ArrayList<>();
                list.add(r);
                records.put(key, list);
            }
        }

        Long courseId = null;
        Long accountId = null;
        Long duration = null;
        List<Recorde> accountRecord = null;
        Date date = new Date();

        // 查询学员是否完成这些课程
        Map<String, Boolean> map = courseClient.isCourseFinished(records.keySet(), true);

        for (Map.Entry<String, List<Recorde>> entry : records.entrySet()) {
            duration = 0L;
            courseId = Long.valueOf(entry.getKey().split("_")[0]);
            accountId = Long.valueOf(entry.getKey().split("_")[1]);
            accountRecord = entry.getValue();
            if (!CollectionUtils.isEmpty(accountRecord)) {
                for (Recorde recorde : accountRecord) {
                    duration += recorde.getDuration() == null ? 0 : recorde.getDuration();
                }
            }
            LOG.info("发送消息：--------------------------------------------");
            //LOG.info("newRecords：{}", newRecords);
            LOG.info("map：{}", map);
            toProject(courseId, accountId, duration,
                    map.get(entry.getKey()) != null ? map.get(entry.getKey()) : false, accountRecord.get(0).getSiteId(), date);
        }

    }

    private Long stringToLong(String str) {
        Long l = 0L;
        try {
            l = Long.valueOf(str);
            return l;
        } catch (Exception e) {
            LOG.error("id:" + str + " 转换失败" + e.getMessage());
        }
        return l;
    }

    /**
     * 组装该章节的完成状态
     *
     * @param chapterId
     * @param accountId
     * @param duration
     * @return
     */
    private Integer getCompleted(Long chapterId, Long accountId, Integer duration) {

        Chapter chapter = null;
        if (allChaptersMap.containsKey(chapterId)) {
            chapter = allChaptersMap.get(chapterId);
        } else {
            return 0;
        }

        Material material = null;
        if (allMaterialsMap.containsKey(chapter.getMaterialId())) {
            material = allMaterialsMap.get(chapter.getMaterialId());
        } else {
            return 0;
        }

        Integer hour = material.getTextHour();
        Integer minute = material.getTextMinute();
        Integer second = material.getTextSecond();

        Integer total = 0;
        if (!StringUtils.isEmpty(hour)) {
            total = total + hour * 3600;
        }
        if (!StringUtils.isEmpty(minute)) {
            total = total + minute * 60;
        }
        if (!StringUtils.isEmpty(second)) {
            total = total + second;
        }

        //课程的完成规则
        //1点进去就算完成
        //2超过完成时间才算完成

        Integer completed = 0;
        Course course = allCoursesMap.get(chapter.getCourseId());
        if (allCoursesMap.containsKey(chapter.getCourseId())) {
            switch (material.getType()) {
                case 1:
                    if (!StringUtils.isEmpty(course.getAudioComplete())) {
                        if (course.getAudioComplete() == 0) {
                            completed = 1;
                        } else {
                            completed = getCompletedStatus(chapterId, accountId, total, duration);
                        }
                    }
                    break;
                case 2:
                    if (!StringUtils.isEmpty(course.getVideoComplete())) {
                        if (course.getVideoComplete() == 0) {
                            completed = 1;
                        } else {
                            completed = getCompletedStatus(chapterId, accountId, total, duration);
                        }
                    }

                    break;
                case 3:
                    if (!StringUtils.isEmpty(course.getDocComplete())) {
                        if (course.getDocComplete() == 0) {
                            completed = 1;
                        } else {
                            completed = getCompletedStatus(chapterId, accountId, total, duration);
                        }
                    }

                    break;
                case 4:
                    if (!StringUtils.isEmpty(course.getRichComplete())) {
                        if (course.getRichComplete() == 0) {
                            completed = 1;
                        } else {
                            completed = getCompletedStatus(chapterId, accountId, total, duration);
                        }
                    }
                    break;
                case 5:
                    //素材为Scrom的在 页面停留10算完成
                    completed = 0;
                    break;
                default:
                    break;
            }
        }
        return completed;
    }

    /**
     * 根据章节用户id 获得课程学习记录时间 用于判定是否课程完成
     *
     * @param chapterId 章节id
     * @param accountId 账号id
     * @param total     学习课程所需总数
     * @param duration  还未入库的学习时长
     * @return
     */
    private Integer getCompletedStatus(Long chapterId, Long accountId, Integer total, Integer duration) {
        Integer result = 0;
        if (allRecordeDurationMap.containsKey(accountId.toString() + chapterId.toString())) {
            Integer recordeDuration = allRecordeDurationMap.get(accountId.toString() + chapterId.toString());
            if (!StringUtils.isEmpty(recordeDuration)) {
                if (recordeDuration + duration >= total) {
                    result = 1;
                } else {
                    result = 0;
                }
            } else {
                if (duration >= total) {
                    result = 1;
                }
            }
        }
        return result;
    }

    private void getPoint(Long chapterId, Long accountId) {
        try {
            if (allChaptersMap.containsKey(chapterId)) {
                Chapter chapter = allChaptersMap.get(chapterId);
                if (allMaterialsMap.containsKey(chapter.getMaterialId())) {
                    Material material = allMaterialsMap.get(chapter.getMaterialId());
                    PointParamVO pointParamVO = new PointParamVO();
                    //事件名称
                    pointParamVO.setEventName("pointCourse");
                    //事件类型
                    pointParamVO.setActivityType("课程章节完成发放积分");
                    //学习活动来源
                    pointParamVO.setActivitySource("课程");
                    //学习活动名
                    StringBuilder activityName = new StringBuilder();

                    if (allCoursesMap.containsKey(chapter.getCourseId())) {
                        Course course = allCoursesMap.get(chapter.getCourseId());
                        activityName.append(course.getName());
                        activityName.append(":");
                    }
                    activityName.append(chapter.getName());
                    pointParamVO.setActivityName(activityName.toString());
                    //发放条件
                    pointParamVO.setReleaseCondition("课程章节发放完成");
                    //发放规则
                    pointParamVO.setReleaseRules("按照章节获取取积分");
                    //积分创建时间
                    pointParamVO.setCreatePointTime(new Date());
                    //积分来源ID，（业务ID主键）
                    pointParamVO.setSourceId(chapter.getId());
                    //积分数量，  正数 加积分，  负数减积分
                    pointParamVO.setOperatingPoint(chapter.getScore());
                    //学员ID
                    pointParamVO.setAccountId(accountId);
                    //学员名称
                    pointParamVO.setAccountName("");
                    pointParamVO.setSiteId(material.getSiteId());
                    pointParamVO.setCompanyId(material.getCompanyId());
                    pointParamVO.setOrgId(material.getOrgId());
                    //调用积分接口添加积分
                    String sitePointId = pointRedisFeignClients.addPointRedis(pointParamVO);
                    if (null != sitePointId) {
                        //发送积分获取消息
                        amqpTemplate.convertAndSend("course", sitePointId);
                    }
                }
            }
        } catch (Exception e) {
            LOG.error("章节" + chapterId + "积分消息发送失败！");
        }


    }

    /**
     * 给项目发送课程完成消息
     */
    private void toProject(Long courseId, Long accountId, Long duration, boolean overed, Long siteId, Date date) {
        try {
            cloudEventPublisher.publish(QueueConstant.TRAINING_PROJECT_EVENT_QUEUE,
                    new EventWrapper<TrainingProjectEvent>(courseId,
                            TrainingProjectEvent.getInstanceCourse(courseId, accountId, date, duration, overed, siteId)));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    @Override
    public void interrupt() throws UnableToInterruptJobException {

    }

    /**
     * 获取数据库中已存在的学习记录的 sessionKey
     *
     * @param recorde
     * @return
     */
    private String makeSessionKey(Recorde recorde) {
        String sessionkey = "";
        if (recorde.getAccountId() == null) {
            sessionkey = recorde.getSessionId();
        } else {
            sessionkey = recorde.getAccountId().toString();
            if (recorde.getCourseId() != null) {
                sessionkey += "_" + recorde.getCourseId();
            }
            if (recorde.getChapterId() != null) {
                sessionkey += "_" + recorde.getChapterId();
            }
        }

        return sessionkey;
    }

    /**
     * 获取本次学习记录的 sessionKey
     *
     * @param studyLog
     * @return
     */
    private String makeSessionKey(StudyLog studyLog) {
        String sessionkey = "";
        if (studyLog.getAccountId() == null) {
            sessionkey = studyLog.getSessionId();
        } else {
            sessionkey = studyLog.getAccountId().toString();
            if (studyLog.getCourseId() != null) {
                sessionkey += "_" + studyLog.getCourseId();
            }
            if (studyLog.getChapterId() != null) {
                sessionkey += "_" + studyLog.getChapterId();
            }
        }
        return sessionkey;
    }
}
