package com.fulan.application.handle.quartz.job;

import com.fulan.application.cache.CacheNamespace;
import com.fulan.application.handle.quartz.BaseJob;
import com.fulan.application.system.remote.OrganizationClient;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.collections.CollectionUtils;
import org.quartz.InterruptableJob;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.UnableToInterruptJobException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import redis.clients.jedis.util.SafeEncoder;

import java.util.ArrayList;
import java.util.Set;

/**
 * @ClassName OrganizationRedisJob
 * @Description 定时重置部门索引
 * @Author shengchenglong
 * @DATE 2019-04-10 14:59
 * @Version 1.0
 */
@Log4j2
public class OrganizationRedisJob implements BaseJob, InterruptableJob {

    private static final Logger LOGGER = LoggerFactory.getLogger(OrganizationRedisJob.class);

    @Autowired
    private OrganizationClient organizationClient;
    @Autowired
    private RedisTemplate redisTemplate;

    @Override
    public void execute(JobExecutionContext context) throws JobExecutionException {

        try {
            LOGGER.info("----------------开始 redis 队列 重建部门索引 ---------------------");
            boolean continueFlag = true;
            Set<byte[]> set = null;
            Long companyId = null;
            while (continueFlag) {
                set = redisTemplate.getConnectionFactory().getConnection().zRange(SafeEncoder.encode(CacheNamespace.ORG_INDEX_INIT_KEY), 0, 10);
                if (CollectionUtils.isEmpty(set)) {
                    continueFlag = false;
                } else {
                    for (byte[] o : set) {
                        if (o == null) {
                            continue;
                        }
                        companyId = Long.valueOf(SafeEncoder.encode(o));
                        LOGGER.info("----------------开始 redis 队列 重建部门索引 companyId: ---------------------" + companyId);
                        organizationClient.rebuildOrgIndexJob(companyId);
                        LOGGER.info("----------------完成 redis 队列 重建部门索引 companyId: ---------------------" + companyId);
                        redisTemplate.getConnectionFactory().getConnection().zRem(SafeEncoder.encode(CacheNamespace.ORG_INDEX_INIT_KEY), o);
                    }
                }
            }
            LOGGER.info("----------------结束 redis 队列 重建部门索引 ---------------------");
        } catch (Exception e) {
            e.printStackTrace();
            log.error("");
        }
    }

    @Override
    public void interrupt() throws UnableToInterruptJobException {

    }
}
